@@ -15,6 +15,7 @@
 #include <unistd.h>
 
 #include "kill.h"
+#include "meminfo.h"
 #include "msg.h"
 
 #define BADNESS_PREFER 300
@@ -24,13 +25,6 @@ extern int enable_debug;
 extern long page_size;
 void sanitize(char* s);
 
-struct procinfo {
-    int oom_score;
-    int oom_score_adj;
-    unsigned long VmRSSkiB;
-    int exited;
-};
-
 static int isnumeric(char* str)
 {
     int i = 0;
@@ -55,72 +49,64 @@ static void maybe_notify(char* notif_command, char* notif_args)
     if (!notif_command)
         return;
 
-    char notif[600];
+    char notif[PATH_MAX + 2000];
     snprintf(notif, sizeof(notif), "%s %s", notif_command, notif_args);
     if (system(notif) != 0)
         warn("system('%s') failed: %s\n", notif, strerror(errno));
 }
 
-const char* const fopen_msg = "fopen %s failed: %s\n";
-
-/* Read /proc/pid/{oom_score, oom_score_adj, statm}
- * Caller must ensure that we are already in the /proc/ directory
+/*
+ * Send the selected signal to "pid" and wait for the process to exit
+ * (max 10 seconds)
  */
-static struct procinfo get_process_stats(int pid)
+int kill_wait(const poll_loop_args_t args, pid_t pid, int sig)
 {
-    char buf[256];
-    FILE* f;
-    struct procinfo p = { 0 };
-
-    // Read /proc/[pid]/oom_score
-    snprintf(buf, sizeof(buf), "%d/oom_score", pid);
-    f = fopen(buf, "r");
-    if (f == NULL) {
-        printf(fopen_msg, buf, strerror(errno));
-        p.exited = 1;
-        return p;
+    meminfo_t m = { 0 };
+    const int poll_ms = 100;
+    int res = kill(pid, sig);
+    if (res != 0) {
+        return res;
     }
-    if (fscanf(f, "%d", &(p.oom_score)) < 1)
-        warn("fscanf() oom_score failed: %s\n", strerror(errno));
-    fclose(f);
-
-    // Read /proc/[pid]/oom_score_adj
-    snprintf(buf, sizeof(buf), "%d/oom_score_adj", pid);
-    f = fopen(buf, "r");
-    if (f == NULL) {
-        printf(fopen_msg, buf, strerror(errno));
-        p.exited = 1;
-        return p;
+    /* signal 0 does not kill the process. Don't wait for it to exit */
+    if (sig == 0) {
+        return 0;
     }
-    if (fscanf(f, "%d", &(p.oom_score_adj)) < 1)
-        warn("fscanf() oom_score_adj failed: %s\n", strerror(errno));
-
-    fclose(f);
-
-    // Read VmRSS from /proc/[pid]/statm (in pages)
-    snprintf(buf, sizeof(buf), "%d/statm", pid);
-    f = fopen(buf, "r");
-    if (f == NULL) {
-        printf(fopen_msg, buf, strerror(errno));
-        p.exited = 1;
-        return p;
+    for (int i = 0; i < 100; i++) {
+        float secs = ((float)i) * poll_ms / 1000;
+        // We have sent SIGTERM but now have dropped below SIGKILL limits.
+        // Escalate to SIGKILL.
+        if (sig != SIGKILL) {
+            m = parse_meminfo();
+            if (enable_debug) {
+                print_mem_stats(0, m);
+            }
+            if (m.MemAvailablePercent <= args.mem_kill_percent && m.SwapFreePercent <= args.swap_kill_percent) {
+                sig = SIGKILL;
+                res = kill(pid, sig);
+                // kill first, print after
+                warn("escalating to SIGKILL after %.1f seconds\n", secs);
+                if (res != 0) {
+                    return res;
+                }
+            }
+        } else if (enable_debug) {
+            m = parse_meminfo();
+            print_mem_stats(0, m);
+        }
+        if (!is_alive(pid)) {
+            warn("process exited after %.1f seconds\n", secs);
+            return 0;
+        }
+        usleep(poll_ms * 1000);
     }
-    if (fscanf(f, "%*u %lu", &(p.VmRSSkiB)) < 1) {
-        warn("fscanf() vm_rss failed: %s\n", strerror(errno));
-    }
-    // Value is in pages. Convert to kiB.
-    p.VmRSSkiB = p.VmRSSkiB * page_size / 1024;
-
-    fclose(f);
-
-    return p;
+    errno = ETIME;
+    return -1;
 }
 
 /*
  * Find the process with the largest oom_score and kill it.
- * See trigger_kernel_oom() for the reason why this is done in userspace.
  */
-void userspace_kill(poll_loop_args_t args, int sig)
+void kill_largest_process(const poll_loop_args_t args, int sig)
 {
     struct dirent* d;
     char buf[256];
@@ -128,8 +114,7 @@ void userspace_kill(poll_loop_args_t args, int sig)
     int victim_pid = 0;
     int victim_badness = 0;
     unsigned long victim_vm_rss = 0;
-    char name[PATH_MAX];
-    char victim_name[PATH_MAX] = { 0 };
+    char victim_name[256] = { 0 };
     struct procinfo p;
     int badness;
     struct timespec t0 = { 0 }, t1 = { 0 };
@@ -138,10 +123,15 @@ void userspace_kill(poll_loop_args_t args, int sig)
         clock_gettime(CLOCK_MONOTONIC, &t0);
     }
 
-    rewinddir(args.procdir);
+    // main() makes sure that we are in /proc
+    DIR* procdir = opendir(".");
+    if (procdir == NULL) {
+        fatal(5, "Could not open /proc: %s", strerror(errno));
+    }
+
     while (1) {
         errno = 0;
-        d = readdir(args.procdir);
+        d = readdir(procdir);
         if (d == NULL) {
             if (errno != 0)
                 warn("userspace_kill: readdir error: %s", strerror(errno));
@@ -174,17 +164,26 @@ void userspace_kill(poll_loop_args_t args, int sig)
         if (args.ignore_oom_score_adj && p.oom_score_adj > 0)
             badness -= p.oom_score_adj;
 
-        name[0] = 0;
-        snprintf(buf, sizeof(buf), "%d/stat", pid);
-        FILE* stat = fopen(buf, "r");
-        if (stat) {
-            if (fscanf(stat, "%*d (%[^)]s", name) < 1)
-                warn("fscanf() stat name failed: %s\n", strerror(errno));
-
-            fclose(stat);
+        char name[256] = { 0 };
+        snprintf(buf, sizeof(buf), "%d/comm", pid);
+        FILE* comm = fopen(buf, "r");
+        if (comm) {
+            const int TASK_COMM_LEN = 16;
+            int n = fread(name, 1, TASK_COMM_LEN, comm);
+            // Strip trailing newline
+            if (n > 1) {
+                name[n - 1] = 0;
+            } else {
+                warn("reading %s failed: %s", buf, strerror(errno));
+            }
+            fclose(comm);
         } else {
-            warn("could not read process name: %s", strerror(errno));
+            warn("could not open %s: %s", buf, strerror(errno));
         }
+        // The kernel truncates /proc/[pid]/comm at 16 bytes. This
+        // may result in broken utf8, which causes problems when
+        // viewing the logs. Fix it.
+        fix_truncated_utf8(name);
 
         if (args.prefer_regex && regexec(args.prefer_regex, name, (size_t)0, NULL, 0) == 0) {
             badness += BADNESS_PREFER;
@@ -200,17 +199,18 @@ void userspace_kill(poll_loop_args_t args, int sig)
             victim_pid = pid;
             victim_badness = badness;
             victim_vm_rss = p.VmRSSkiB;
-            strncpy(victim_name, name, sizeof(victim_name) - 1);
+            strncpy(victim_name, name, sizeof(victim_name));
             if (enable_debug)
                 printf("    ^ new victim (higher badness)\n");
         } else if (badness == victim_badness && p.VmRSSkiB > victim_vm_rss) {
             victim_pid = pid;
             victim_vm_rss = p.VmRSSkiB;
-            strncpy(victim_name, name, sizeof(victim_name) - 1);
+            strncpy(victim_name, name, sizeof(victim_name));
             if (enable_debug)
                 printf("    ^ new victim (higher vm_rss)\n");
         }
     } // end of while(1) loop
+    closedir(procdir);
 
     if (victim_pid == 0) {
         warn("Could not find a process to kill. Sleeping 1 second.\n");
@@ -220,20 +220,27 @@ void userspace_kill(poll_loop_args_t args, int sig)
         return;
     }
 
-    // sig == 0 is used as a self-test during startup. Don't notifiy the user.
-    if (sig != 0) {
-        warn("Killing process '%s' with signal %d, pid: %d, badness: %d, VmRSS: %lu MiB\n",
-            victim_name, sig, victim_pid, victim_badness, victim_vm_rss / 1024);
-    }
-
-    int res = kill(victim_pid, sig);
-
     if (enable_debug) {
         clock_gettime(CLOCK_MONOTONIC, &t1);
         long delta = (t1.tv_sec - t0.tv_sec) * 1000000 + (t1.tv_nsec - t0.tv_nsec) / 1000;
-        printf("selecting victim and sending signal took %ld.%03ld ms\n", delta / 1000, delta % 1000);
+        printf("selecting victim took %ld.%03ld ms\n", delta / 1000, delta % 1000);
     }
 
+    char* sig_name = "?";
+    if (sig == SIGTERM) {
+        sig_name = "SIGTERM";
+    } else if (sig == SIGKILL) {
+        sig_name = "SIGKILL";
+    }
+    // sig == 0 is used as a self-test during startup. Don't notifiy the user.
+    if (sig != 0) {
+        warn("sending %s to process %d \"%s\": badness %d, VmRSS %lu MiB\n",
+            sig_name, victim_pid, victim_name, victim_badness, victim_vm_rss / 1024);
+    }
+
+    int res = kill_wait(args, victim_pid, sig);
+    int saved_errno = errno;
+
     // Send the GUI notification AFTER killing a process. This makes it more likely
     // that there is enough memory to spawn the notification helper.
     if (sig != 0) {
@@ -245,13 +252,20 @@ void userspace_kill(poll_loop_args_t args, int sig)
         maybe_notify(args.notif_command, notif_args);
     }
 
-    // Killing the process may have failed because we are not running as root.
-    // In that case, trying again in 100ms will just yield the same error.
-    // Throttle ourselves to not spam the log.
-    if (sig != 0 && res != 0) {
-        warn("kill() failed: %s. Sleeping 1 second\n", strerror(errno));
+    if (sig == 0) {
+        return;
+    }
+
+    if (res != 0) {
+        warn("kill failed: %s\n", strerror(saved_errno));
         maybe_notify(args.notif_command,
-            "-i dialog-error 'earlyoom' 'Error: Failed to kill process. Sleeping 1 second.'");
-        sleep(1);
+            "-i dialog-error 'earlyoom' 'Error: Failed to kill process'");
+        // Killing the process may have failed because we are not running as root.
+        // In that case, trying again in 100ms will just yield the same error.
+        // Throttle ourselves to not spam the log.
+        if (saved_errno == EPERM) {
+            warn("sleeping 1 second\n");
+            sleep(1);
+        }
     }
 }
