@@ -5,8 +5,6 @@
 #include <stdbool.h>
 
 typedef struct {
-    /* directory file handle to /proc */
-    DIR* procdir;
     /* if the available memory AND swap goes below these percentages,
      * we start killing processes */
     int mem_term_percent;
@@ -24,6 +22,6 @@ typedef struct {
     int report_interval_ms;
 } poll_loop_args_t;
 
-void userspace_kill(poll_loop_args_t args, int sig);
+void kill_largest_process(poll_loop_args_t args, int sig);
 
 #endif
