@@ -15,10 +15,19 @@
 #include <sys/resource.h>
 #include <unistd.h>
 
+#include "globals.h"
 #include "kill.h"
 #include "meminfo.h"
 #include "msg.h"
 
+/* Don't fail compilation if the user has an old glibc that
+ * does not define MCL_ONFAULT. The kernel may still be recent
+ * enough to support the flag.
+ */
+#ifndef MCL_ONFAULT
+#define MCL_ONFAULT 4
+#endif
+
 /* Arbitrary identifiers for long options that do not have a short
  * version */
 enum {
@@ -27,12 +36,8 @@ enum {
 };
 
 static int set_oom_score_adj(int);
-static void print_mem_stats(bool lowmem, const meminfo_t m);
 static void poll_loop(const poll_loop_args_t args);
 
-int enable_debug = 0;
-long page_size = 0;
-
 int main(int argc, char* argv[])
 {
     poll_loop_args_t args = {
@@ -60,11 +65,6 @@ int main(int argc, char* argv[])
         fatal(4, "Could not cd to /proc: %s", strerror(errno));
     }
 
-    args.procdir = opendir(".");
-    if (args.procdir == NULL) {
-        fatal(5, "Could not open /proc: %s", strerror(errno));
-    }
-
     meminfo_t m = parse_meminfo();
 
     int c;
@@ -87,26 +87,42 @@ int main(int argc, char* argv[])
             break;
         case 'm':
             // Use 99 as upper limit. Passing "-m 100" makes no sense.
-            tuple = parse_term_kill_tuple("-m", optarg, 99, 15);
+            tuple = parse_term_kill_tuple(optarg, 99);
+            if (strlen(tuple.err)) {
+                fatal(15, "-m: %s", tuple.err);
+            }
             args.mem_term_percent = tuple.term;
             args.mem_kill_percent = tuple.kill;
             have_m = 1;
             break;
         case 's':
             // Using "-s 100" is a valid way to ignore swap usage
-            tuple = parse_term_kill_tuple("-s", optarg, 100, 16);
+            tuple = parse_term_kill_tuple(optarg, 100);
+            if (strlen(tuple.err)) {
+                fatal(16, "-s: %s", tuple.err);
+            }
             args.swap_term_percent = tuple.term;
             args.swap_kill_percent = tuple.kill;
             have_s = 1;
             break;
         case 'M':
-            tuple = parse_term_kill_tuple("-M", optarg, m.MemTotalKiB * 100 / 99, 15);
+            tuple = parse_term_kill_tuple(optarg, m.MemTotalKiB * 100 / 99);
+            if (strlen(tuple.err)) {
+                fatal(15, "-M: %s", tuple.err);
+            }
             args.mem_term_percent = 100 * tuple.term / m.MemTotalKiB;
             args.mem_kill_percent = 100 * tuple.kill / m.MemTotalKiB;
             have_M = 1;
             break;
         case 'S':
-            tuple = parse_term_kill_tuple("-S", optarg, m.SwapTotalKiB * 100 / 99, 16);
+            if (m.SwapTotalKiB == 0) {
+                warn("warning: -S: total swap is zero, using default percentages\n");
+                break;
+            }
+            tuple = parse_term_kill_tuple(optarg, m.SwapTotalKiB * 100 / 99);
+            if (strlen(tuple.err)) {
+                fatal(16, "-S: %s", tuple.err);
+            }
             args.swap_term_percent = 100 * tuple.term / m.SwapTotalKiB;
             args.swap_kill_percent = 100 * tuple.kill / m.SwapTotalKiB;
             have_S = 1;
@@ -150,25 +166,32 @@ int main(int argc, char* argv[])
             break;
         case 'h':
             fprintf(stderr,
-                "Usage: earlyoom [OPTION]...\n"
+                "Usage: %s [OPTION]...\n"
                 "\n"
-                "  -m PERCENT[,KILL_PERCENT] set available memory minimum to PERCENT of total (default 10 %%).\n"
-                "                            earlyoom sends SIGTERM once below PERCENT, then SIGKILL once below\n"
-                "                            KILL_PERCENT (default PERCENT/2).\n"
-                "  -s PERCENT[,KILL_PERCENT] set free swap minimum to PERCENT of total (default 10 %%)\n"
+                "  -m PERCENT[,KILL_PERCENT] set available memory minimum to PERCENT of total\n"
+                "                            (default 10 %%).\n"
+                "                            earlyoom sends SIGTERM once below PERCENT, then\n"
+                "                            SIGKILL once below KILL_PERCENT (default PERCENT/2).\n"
+                "  -s PERCENT[,KILL_PERCENT] set free swap minimum to PERCENT of total (default\n"
+                "                            10 %%).\n"
+                "                            Note: both memory and swap must be below minimum for\n"
+                "                            earlyoom to act.\n"
                 "  -M SIZE[,KILL_SIZE]       set available memory minimum to SIZE KiB\n"
                 "  -S SIZE[,KILL_SIZE]       set free swap minimum to SIZE KiB\n"
-                "  -i                        user-space oom killer should ignore positive oom_score_adj values\n"
+                "  -i                        user-space oom killer should ignore positive\n"
+                "                            oom_score_adj values\n"
                 "  -n                        enable notifications using \"notify-send\"\n"
                 "  -N COMMAND                enable notifications using COMMAND\n"
                 "  -d                        enable debugging messages\n"
                 "  -v                        print version information and exit\n"
-                "  -r INTERVAL               memory report interval in seconds (default 1), set to 0 to\n"
-                "                            disable completely\n"
-                "  -p                        set niceness of earlyoom to -20 and oom_score_adj to -1000\n"
+                "  -r INTERVAL               memory report interval in seconds (default 1), set\n"
+                "                            to 0 to disable completely\n"
+                "  -p                        set niceness of earlyoom to -20 and oom_score_adj to\n"
+                "                            -1000\n"
                 "  --prefer REGEX            prefer killing processes matching REGEX\n"
                 "  --avoid REGEX             avoid killing processes matching REGEX\n"
-                "  -h, --help                this help text\n");
+                "  -h, --help                this help text\n",
+                argv[0]);
             exit(0);
         case '?':
             fprintf(stderr, "Try 'earlyoom --help' for more information.\n");
@@ -216,43 +239,34 @@ int main(int argc, char* argv[])
     }
 
     // Print memory limits
-    fprintf(stderr, "mem  total: %4d MiB, sending SIGTERM at %2d %%, SIGKILL at %2d %%\n",
-        m.MemTotalMiB, args.mem_term_percent, args.mem_kill_percent);
-    fprintf(stderr, "swap total: %4d MiB, sending SIGTERM at %2d %%, SIGKILL at %2d %%\n",
-        m.SwapTotalMiB, args.swap_term_percent, args.swap_kill_percent);
+    fprintf(stderr, "mem total: %4d MiB, swap total: %4d MiB\n",
+        m.MemTotalMiB, m.SwapTotalMiB);
+    fprintf(stderr, "sending SIGTERM when mem <= %2d %% and swap <= %2d %%,\n",
+        args.mem_term_percent, args.swap_term_percent);
+    fprintf(stderr, "        SIGKILL when mem <= %2d %% and swap <= %2d %%\n",
+        args.mem_kill_percent, args.swap_kill_percent);
 
     /* Dry-run oom kill to make sure stack grows to maximum size before
      * calling mlockall()
      */
-    userspace_kill(args, 0);
+    if (enable_debug)
+        printf("dry-running kill_largest_process()...\n");
+    kill_largest_process(args, 0);
 
-    if (mlockall(MCL_CURRENT | MCL_FUTURE) != 0)
+    int err = mlockall(MCL_CURRENT | MCL_FUTURE | MCL_ONFAULT);
+    // kernels older than 4.4 don't support MCL_ONFAULT. Retry without it.
+    if (err != 0) {
+        err = mlockall(MCL_CURRENT | MCL_FUTURE);
+    }
+    if (err != 0) {
         perror("Could not lock memory - continuing anyway");
+    }
 
     // Jump into main poll loop
     poll_loop(args);
     return 0;
 }
 
-/* Print a status line like
- *   mem avail: 5259 MiB (67 %), swap free: 0 MiB (0 %)"
- * to the fd passed in out_fd.
- */
-static void print_mem_stats(bool lowmem, const meminfo_t m)
-{
-    int (*out_func)(const char* fmt, ...) = &printf;
-    if (lowmem) {
-        out_func = &warn;
-    }
-    out_func("mem avail: %4d of %4d MiB (%2d %%), swap free: %4d of %4d MiB (%2d %%)\n",
-        m.MemAvailableMiB,
-        m.MemTotalMiB,
-        m.MemAvailablePercent,
-        m.SwapFreeMiB,
-        m.SwapTotalMiB,
-        m.SwapFreePercent);
-}
-
 // Returns errno (success = 0)
 static int set_oom_score_adj(int oom_score_adj)
 {
@@ -314,35 +328,26 @@ static int sleep_time_ms(const poll_loop_args_t* args, const meminfo_t* m)
 static void poll_loop(const poll_loop_args_t args)
 {
     meminfo_t m = { 0 };
+    // Print a a memory report when this reaches zero. We start at zero so
+    // we print the first report immediately.
     int report_countdown_ms = 0;
-    // extra time to sleep after a kill
-    const int cooldown_ms = 200;
 
     while (1) {
+        int sig = 0;
         m = parse_meminfo();
-
-        if (m.MemAvailablePercent <= args.mem_term_percent && m.SwapFreePercent <= args.swap_term_percent) {
-            int sig = 0;
-            if (m.MemAvailablePercent <= args.mem_kill_percent && m.SwapFreePercent <= args.swap_kill_percent) {
-                warn("Low memory! At or below sigkill limits (mem: %d %%, swap: %d %%)\n",
-                    args.mem_kill_percent, args.swap_kill_percent);
-                sig = SIGKILL;
-            } else {
-                warn("Low Memory! At or below sigterm limits (mem: %d %%, swap: %d %%)\n",
-                    args.mem_term_percent, args.swap_term_percent);
-                sig = SIGTERM;
-            }
+        if (m.MemAvailablePercent <= args.mem_kill_percent && m.SwapFreePercent <= args.swap_kill_percent) {
             print_mem_stats(1, m);
-            userspace_kill(args, sig);
-            // With swap enabled, the kernel seems to need more than 100ms to free the memory
-            // of the killed process. This means that earlyoom would immediately kill another
-            // process. Sleep a little extra to give the kernel time to free the memory.
-            // (Yes, this will sleep even if the kill has failed. Does no harm and keeps the
-            // code simple.)
-            if (m.SwapTotalMiB > 0) {
-                usleep(cooldown_ms * 1000);
-                report_countdown_ms -= cooldown_ms;
-            }
+            warn("low memory! at or below SIGKILL limits: mem %d %%, swap %d %%\n",
+                args.mem_kill_percent, args.swap_kill_percent);
+            sig = SIGKILL;
+        } else if (m.MemAvailablePercent <= args.mem_term_percent && m.SwapFreePercent <= args.swap_term_percent) {
+            print_mem_stats(1, m);
+            warn("low memory! at or below SIGTERM limits: mem %d %%, swap %d %%\n",
+                args.mem_term_percent, args.swap_term_percent);
+            sig = SIGTERM;
+        }
+        if (sig) {
+            kill_largest_process(args, sig);
         } else if (args.report_interval_ms && report_countdown_ms <= 0) {
             print_mem_stats(0, m);
             report_countdown_ms = args.report_interval_ms;
